/*
 * ============================================================================
 * (C) Copyright IBM Corporation 2006, 2007, 2008.  All Rights Reserved.
 * 
 * You may only copy, modify, and distribute these samples internally.
 * These samples have not been tested under all conditions and are provided
 * to you by IBM without obligation of support of any kind.
 *
 * IBM PROVIDES THESE SAMPLES "AS IS" SUBJECT TO ANY STATUTORY WARRANTIES THAT
 * CANNOT BE EXCLUDED. IBM MAKES NO WARRANTIES OR CONDITIONS, EITHER EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OR
 * CONDITIONS OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND
 * NON-INFRINGEMENT REGARDING THESE SAMPLES OR TECHNICAL SUPPORT, IF ANY.
 * ============================================================================
 */
 
def onList() {
    def dataManager = zero.data.groovy.Manager.create('books_db')
    def rs
    
    // retrieve title and/or author from query string to filter results, if present
    def titleString = request.params.title[]
    def authorString = request.params.author[]

    if (titleString) {
        def title = '%' + titleString + '%'
        rs = dataManager.queryList("SELECT * FROM books WHERE title LIKE ${title} ORDER BY UPDATED DESC")
    } else if (authorString) {
        def author = '%' + authorString + '%'
        rs = dataManager.queryList("SELECT * FROM books WHERE authorname LIKE ${author} ORDER BY UPDATED DESC")
    } else {
        rs = dataManager.queryArray('SELECT * FROM books ORDER BY UPDATED DESC')
    }
    
    // render the result set as an atom feed
    request.view='atom'
    request.atom.output = rs 
    render() 
}

def onCreate() {
	// parse the incoming atom entry using Abdera
    def entry = org.apache.abdera.Abdera.getNewParser().parse(request.input[]).getRoot()
    
    def updated = new java.sql.Timestamp(System.currentTimeMillis())
    
    if (entry) {
        def dataManager = zero.data.groovy.Manager.create('books_db')   
        
        def catTerm
        def catLabel
        
        if (entry.getCategories().size() > 0)   {                        
            catTerm = entry.getCategories().get(0).getTerm()
            catLabel = entry.getCategories().get(0).getLabel()
        } 

        // insert new entry into the database
        def id = dataManager.insert('INSERT INTO books (TITLE, AUTHORNAME, CONTENT, CONTENTTYPE, UPDATED, CATEGORYTERM, CATEGORYLABEL) VALUES (?1, ?2, ?3, ?4, ?5, ?6, ?7)', 
                ['id'],
                [entry.getTitle(), 
                entry.getAuthor().getName(),        
                entry.getContent().toString(), 
                entry.getContentType().toString(), 
                updated, 
                catTerm,
                catLabel])
 
        // add a location header with the uri of the newly-created resource to the response
        resourceUri = getRequestedUri(false) + '/' + id
        request.headers.out.Location = resourceUri
        
        // add a link back to the newly-created resource into the atom entry
        entry.setId(resourceUri)
        link = entry.addLink(resourceUri)
        link.setAttributeValue('rel', 'edit')

        // render the newly-created atom entry as the response
        request.status = HttpURLConnection.HTTP_NO_CONTENT
        request.view='atom'
        request.atom.output = entry
        render()
    } else {
    	// create request without an entry for a body is not acceptable
    	request.status = HttpURLConnection.HTTP_BAD_REQUEST
    }
}

def onRetrieve() {
	// retrieve the id of the book from the query string
	def id = request.params.booksId[]
	
    def dataManager = zero.data.groovy.Manager.create('books_db')
    def result = dataManager.queryFirst("SELECT * FROM books WHERE id=$id")
    if (result) {
    	// render the result to the response as an atom entry
    	request.view='atom'
    	request.atom.output = result
    	render()
    } else {
    	// no matching entry was found
    	HttpURLConnection.HTTP_NOT_FOUND 
    }
}


def onUpdate() {
	// parse the incoming atom entry using Abdera
    def entry = org.apache.abdera.Abdera.getNewParser().parse(request.input[]).getRoot()
    
    def updated = new java.sql.Timestamp(System.currentTimeMillis()) 
    
    if (entry) {
    	def id = request.params.booksId[]
        
        def dataManager = zero.data.groovy.Manager.create('books_db')
        def catterm = null
        def catlabel = null
        if (entry.getCategories()) {
            catterm = entry.getCategories().get(0).getTerm()
            catlabel = entry.getCategories().get(0).getLabel()
        }
    	// update the entry in the database
        rc = dataManager.update('UPDATE books SET TITLE=?1, AUTHORNAME=?2,  UPDATED=?3, CONTENT=?4, CONTENTTYPE=?5, categoryterm=?6, categorylabel=?7 WHERE ID=?8', 
                                   [entry.getTitle(), 
                                   entry.getAuthor().getName(), 
                                   updated,
                                   entry.getContent().toString(),
                                   entry.getContentType().toString(),
                                   catterm, 
                                   catlabel,
                                   id])

        request.status = HttpURLConnection.HTTP_NO_CONTENT
    } else {
    	// update request without an entry for a body is not acceptable
    	request.status = HttpURLConnection.HTTP_BAD_REQUEST
    }
}


def onDelete() {
     def id = request.params.booksId[]
     def dataManager = zero.data.groovy.Manager.create('books_db')

     // delete the entry from the database
     def rc = dataManager.update("DELETE FROM books WHERE id=$id")
     
     request.status = HttpURLConnection.HTTP_NO_CONTENT
}